import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { Alert, Box, Button, Checkbox, CircularProgress, FormControl, FormLabel, Grid, Input, Modal, ModalDialog, Switch, Typography, useColorScheme, useTheme, } from "@mui/joy";
import { useMediaQuery } from "@mui/material";
import { NerdCircleColorPicker } from "../nerdCircleColorPicker";
import { Suspense, useEffect, useMemo, useState } from "react";
import { useTranslation } from "react-i18next";
import nerdUII18n from "../i18n";
import { Report } from "@mui/icons-material";
import { Searchable } from "./searchable";
import { lazy } from "react";
const CodeMirror = lazy(() => import("@uiw/react-codemirror"));
const getJSONLang = async () => {
    const json = await import("@codemirror/lang-json");
    return json.jsonLanguage;
};
/**
 *
 * @param {SearchPresetFormProps} props props
 * @returns {ReactElement} SearchPresetForm
 */
export function SearchPresetForm(props) {
    const { searchPreset, onClose, filters, definitions, onNewSearchPreset } = props;
    const { t } = useTranslation("common", { i18n: nerdUII18n });
    const theme = useTheme();
    const matches = useMediaQuery(theme.breakpoints.down("md"));
    const [preset, setPreset] = useState(searchPreset?.id
        ? searchPreset
        : {
            ...searchPreset,
            filter: JSON.stringify(filters, null, 2),
        });
    const [jsonValid, setJsonValid] = useState(true);
    const [raw, setRaw] = useState(false);
    const _filters = useMemo(() => {
        if (preset.filter)
            try {
                return JSON.parse(preset.filter);
            }
            catch {
                return [];
            }
        else
            return [];
    }, [preset.filter]);
    const handleClose = () => {
        onClose();
        setJsonValid(true);
        setRaw(false);
        setPreset(searchPreset?.id
            ? searchPreset
            : {
                ...searchPreset,
                filter: JSON.stringify(filters, null, 2),
            });
    };
    useEffect(() => {
        setPreset(searchPreset?.id
            ? searchPreset
            : {
                ...searchPreset,
                filter: JSON.stringify(filters, null, 2),
            });
    }, [searchPreset]);
    return (_jsx(Modal, { open: Boolean(searchPreset), onClose: handleClose, children: _jsxs(ModalDialog, { "aria-labelledby": "modal-dialog-title", maxWidth: matches ? undefined : 600, layout: matches ? "fullscreen" : "center", children: [_jsx(Typography, { id: "modal-dialog-title", level: "h2", children: preset.id ? "Update Search Preset" : "Create Search Preset" }), _jsxs(Grid, { container: true, spacing: 1, sx: {
                        overflow: "auto",
                    }, children: [_jsx(Grid, { xs: 12, children: _jsxs(FormControl, { children: [_jsx(FormLabel, { children: "Name" }), _jsx(Input, { value: preset.name || "", onChange: (e) => setPreset({ ...preset, name: e.target.value }) })] }) }), _jsx(Grid, { xs: 12, children: _jsx(Checkbox, { label: "Share with others", checked: preset.global || false, onChange: (e) => setPreset({ ...preset, global: e.target.checked }) }) }), !jsonValid ? (_jsx(Grid, { xs: 12, children: _jsx(Alert, { startDecorator: _jsx(Report, {}), color: "danger", children: "Invalid filters" }) })) : null, _jsx(Grid, { xs: 12, children: _jsxs(Box, { sx: {
                                    display: "flex",
                                    position: "relative",
                                    flexDirection: "column",
                                }, children: [_jsx(FormLabel, { sx: {
                                            mb: "6px",
                                            alignSelf: "flex-start",
                                        }, children: "Filters" }), _jsx(Typography, { component: "label", endDecorator: _jsx(Switch, { size: "sm", sx: { ml: 1 }, checked: raw, onChange: (event) => setRaw(event.target.checked) }), sx: {
                                            position: "absolute",
                                            right: 0,
                                            top: 0,
                                        }, children: "Raw" }), raw ? (_jsx(JSONEditor, { preset: preset, setJsonValid: setJsonValid, setPreset: setPreset })) : (_jsx(Box, { sx: {
                                            display: "flex",
                                            flexDirection: "column",
                                            gap: 1,
                                            alignItems: "stretch",
                                            maxHeight: 300,
                                            overflowX: "auto",
                                        }, children: _filters.map((f, i) => (_jsx(Searchable, { index: i, definition: definitions?.find((m) => m.name === f.name), onDelete: (i) => console.log(i), filters: _filters, onFiltersChange: (newFilters) => {
                                                setPreset({
                                                    ...preset,
                                                    filter: JSON.stringify(newFilters, null, 2),
                                                });
                                            } }, i))) }))] }) }), _jsx(Grid, { xs: 12, children: _jsxs(FormControl, { children: [_jsx(FormLabel, { children: "Color" }), _jsx(NerdCircleColorPicker, { value: preset.color || "", onChange: (v) => setPreset({ ...preset, color: v }) })] }) })] }), _jsxs(Box, { sx: {
                        display: "flex",
                        gap: 1,
                        justifyContent: "flex-end",
                        alignItems: "end",
                        pt: 2,
                    }, children: [_jsx(Button, { onClick: handleClose, color: "neutral", variant: "soft", children: t("global.cancel") }), _jsx(Button, { onClick: () => {
                                handleClose();
                                onNewSearchPreset(preset);
                            }, children: searchPreset?.id ? t("global.save") : t("global.create") })] })] }) }));
}
const JSONEditor = ({ preset, setPreset, setJsonValid, }) => {
    const [jsonLang, setJsonLang] = useState(null);
    const { mode } = useColorScheme();
    const checkJson = (json) => {
        try {
            JSON.parse(json);
            setJsonValid(true);
        }
        catch {
            setJsonValid(false);
        }
    };
    useEffect(() => {
        getJSONLang()
            .then((e) => {
            setJsonLang(e);
        })
            .catch((e) => console.error(e));
    }, []);
    return (_jsx(Suspense, { fallback: _jsx(Box, { display: "flex", justifyContent: "center", width: "100%", children: _jsx(CircularProgress, {}) }), children: !jsonLang ? (_jsx(Box, { display: "flex", justifyContent: "center", width: "100%", children: _jsx(CircularProgress, {}) })) : (_jsx(CodeMirror, { value: preset.filter, height: "200px", theme: mode === "dark" ? "dark" : "light", extensions: [jsonLang], onChange: (v) => {
                setPreset({ ...preset, filter: v });
                checkJson(v);
            } })) }));
};
